import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import Qt.labs.qmlmodels

import "../components"
import "./view"

import Artix.Handlers
import Artix.Core
import Artix.Gui
import Artix.Models

/**
 * @brief Контекст возврата
 */
BasicView {
    id: root
    readonly property list<int> columnsWidth: [
        tableView.contentWidth - 330, 80, 250
    ]

    property FormEventFilter eventFilter: FormEventFilter {
        onKeyboardData: (data, rawData, source) => handler.input(data, rawData, source)
        onOk: {
            selectionModel.clearCurrentIndex()
            handler.selectAll()
        }
        onCancel: handler.esc()
    }

    // Делегат отображения текстовой информации
    component TextDelegate: GroupBox {
        id: textDelegate
        Style.class: backQuant && !Utils.weightIsZero(backQuant)
                     ? "backBySaleChoosedDelegate" : "backBySaleBasicDelegate"
        leftPadding: column === 0 ? 24 : 0
        rightPadding: column === tableView.columns - 1 ? 24 : 0
        contentItem: BasicLabel {
            Style.class: "backBySaleDelegate"
            text: display ?? ""
            verticalAlignment: Label.AlignVCenter
            horizontalAlignment: column === 2 ? Label.AlignHCenter : Label.AlignLeft
            elide: Text.ElideRight
        }
        pressed: false
        down: pressed || selectionModel.currentIndex.row === row
        animationDuration: 0
        MouseArea {
            id: ma
            anchors.fill: parent
            onPressedChanged: tableView.rowPressedChanged(row, pressed)
            onClicked: selectionModel.setCurrentIndex(documentModel.index(row, column), ItemSelectionModel.SelectCurrent) // выделить позицию
        }
        Connections {
            target: tableView
            function onRowPressedChanged(row, pressed) {
                if (row === model.row)
                    textDelegate.pressed = pressed
            }
        }
        Line {
            anchors.top: parent.bottom
            width: textDelegate.width
            strokeWidth: 1
            color: Style.parameter("color.grayBorder")
        }
    }
    // Делегат редактирования количества
    component EditDelegate: GroupBox {
        id: editDelegate
        Style.class: "backBySaleBasicDelegate"

        property real quantValue: backQuant ? backQuant : 0.0
        function commintBackQuant() {
            let oldValue = backQuant
            if (oldValue === undefined)
                oldValue = 0.0
            if (!Utils.weightsAreEqual(quantValue, oldValue))
                handler.enter(posNum, quantValue)
            quntityField.forceActiveFocus()
        }
        onQuantValueChanged: commitQuantTimer.restart()
        Timer {
            id: commitQuantTimer
            interval: 1000
            onTriggered: editDelegate.commintBackQuant()
        }
        Connections {
            target: tableView
            function onInstantCommit() {
                commitQuantTimer.stop()
                editDelegate.commintBackQuant()
            }
        }
        property real minValue: 0.0
        property real maxValue: quant
        function update(value) {
            commitQuantTimer.restart()
            let parsed = value.length === 0 ? 0.0 : parseFloat(value)
            if (!parsed) {
                quantValue = 0.0
                return
            }
            if (Utils.weightsAreEqual(parsed, quantValue))
                return
            if (Utils.weightGreater(parsed, maxValue)) {
                quantValue = maxValue
                editDelegate.quantValueChanged()
                return
            }
            if (Utils.weightLess(parsed, minValue)) {
                quantValue = minValue
                editDelegate.quantValueChanged()
                return
            }
            quantValue = value
        }

        down: true
        animationDuration: 0
        leftPadding: column === 0 ? 24 : 0
        rightPadding: column === tableView.columns - 1 ? 24 : 0
        contentItem: RowLayout {
            spacing: 8
            BasicButton {
                Style.class: "backBySaleQuant"
                icon: Style.parameter("icon.minus")
                Layout.preferredWidth: 56
                Layout.preferredHeight: 56
                enabled: Utils.weightGreater(editDelegate.quantValue, 0)
                onClicked: editDelegate.update(quantValue - 1)
            }
            BasicTextField {
                id: quntityField
                Style.class: "backBySaleEditBackQuantity"
                Layout.fillWidth: true
                Layout.preferredHeight: 56
                horizontalAlignment: TextField.AlignHCenter
                verticalAlignment: TextField.AlignVCenter
                hideCursor: true
                fixedCursorPosition: true
                overwriteMode: true
                RegularExpressionValidator { id: dv; regularExpression: new RegExp(/^([1-9]\d{0,6}|0[1-9]{0,1})(\.\d{0,3})/) }
                RegularExpressionValidator { id: iv; regularExpression: new RegExp(/^([1-9]\d{0,6}|0[1-9]{0,1})/) }
                validator: fractional ? dv : iv
                text: fractional ? parseFloat(Utils.weightStr(editDelegate.quantValue))
                                 : editDelegate.quantValue.toFixed(0)
                Component.onCompleted: forceActiveFocus()
                onTextChanged: {
                    // на случай, если ввод остановился на разделяющем символе '.', чтобы символ не удалялся
                    if (text.length > 0 && text.charAt(text.length - 1) === '.')
                        commitQuantTimer.stop()
                    else
                        editDelegate.update(text)
                }
            }
            BasicButton {
                Style.class: "backBySaleQuant"
                icon: Style.parameter("icon.plus")
                Layout.preferredWidth: 56
                Layout.preferredHeight: 56
                enabled: Utils.weightLess(editDelegate.quantValue, quant)
                onClicked: editDelegate.update(quantValue + 1)
            }
        }
        Line {
            anchors.top: parent.bottom
            width: editDelegate.width
            strokeWidth: 1
            color: Style.parameter("color.grayBorder")
        }
    }

    ItemSelectionModel {
        id: selectionModel
        model: DocumentBackModel { id: documentModel }
        onCurrentChanged: tableView.instantCommit()
    }

    contentItem: RowLayout {
        spacing: 0
        // Левая часть
        Rectangle {
            Layout.fillWidth: true
            Layout.fillHeight: true
            color: Style.parameter("color.modalBackground")
            ColumnLayout {
                anchors.fill: parent
                spacing: 0
                BasicLabel {
                    Layout.margins: 24
                    Style.class: "backBySaleTitle"
                    Tr { code: "backBySaleTitle"; defaultMessage: "Выберите позиции для возврата" }
                }
                HorizontalHeaderView {
                    id: headerView
                    required property var model
                    Layout.fillWidth: true
                    Layout.preferredHeight: 30
                    model: documentModel
                    syncView: tableView
                    clip: true
                    resizableColumns: false
                    interactive: false
                    columnWidthProvider: function (column) { return columnsWidth[column] }
                    rowHeightProvider: function (row) { return 30 }
                    delegate: BasicLabel {
                        Style.class: "backBySaleTableHeader"
                        leftPadding: column === 0 ? 24 : 0
                        rightPadding: column === tableView.columns - 1 ? 24 : 0
                        verticalAlignment: Label.AlignVCenter
                        horizontalAlignment: column === 2 ? Label.AlignHCenter : Label.AlignLeft
                        Tr { tr: decoration }

                        // FIXME: модель шлет сигнал modelReset, из-за чего виджет пересоздается.
                        // Но после удаления старый виджет продолжает отображаться на экране, причем в неправильной позиции.
                        // Из-за этого возникает задвоение заголовков. Установка `visible = false` при удалении - костыль
                        Component.onDestruction: () => { visible = false }
                    }
                }
                Line {
                    Layout.fillWidth: true
                    Layout.preferredHeight: strokeWidth
                    strokeWidth: 2
                    color: Style.parameter("color.grayDark")
                }
                TableView {
                    id: tableView
                    Layout.fillWidth: true
                    Layout.fillHeight: true
                    signal rowPressedChanged(int row, bool value)
                    signal instantCommit()
                    model: documentModel
                    rowSpacing: 1
                    contentWidth: width
                    clip: true
                    columnWidthProvider: function (column) { return columnsWidth[column] }
                    rowHeightProvider: function (row) { return 88 }
                    delegate: DelegateChooser {
                        role: "contentType"
                        DelegateChoice {
                            roleValue: DocumentBackModel.ConstantContent
                            delegate: TextDelegate {}
                        }
                        DelegateChoice {
                            roleValue: DocumentBackModel.EditableDelegate
                            delegate: Loader {
                                id: editLoader
                                property bool selected: selectionModel.currentIndex.row === row
                                property Component text: TextDelegate {}
                                property Component edit: EditDelegate {}
                                sourceComponent: selected ? edit : text
                            }
                        }
                    }
                    ScrollBar.vertical: ScrollBar {
                        id: scrollBar
                        Timer {
                            id: scrollTimer
                            interval: 2500
                            onTriggered: parent.active = false
                        }
                        onActiveChanged: scrollTimer.start()
                    }
                }
            }
        }
        // Правая часть
        Item {
            Layout.fillHeight: true
            Layout.preferredWidth: 304
            TopPanel {
                anchors { left: parent.left; top: parent.top; right: parent.right }
                anchors.leftMargin: 24
                hideUserName: true
            }
            ColumnLayout {
                anchors { left: parent.left; right: parent.right; bottom: parent.bottom; margins: 24 }
                spacing: 0
                InlineKeyboard {
                    Layout.fillWidth: true
                    Layout.preferredHeight: 296
                    layoutType: "digits"
                }
                Item { Layout.preferredHeight: 16 }
                BasicButton {
                    Style.class: "backBySaleCancel"
                    Layout.fillWidth: true
                    Layout.preferredHeight: 68
                    Tr { code: "backBySaleCancel"; defaultMessage: "Назад" }
                    onClicked: handler.esc()
                }
                Item { Layout.preferredHeight: 8 }
                BasicButton {
                    Style.class: "backBySaleAll"
                    Layout.fillWidth: true
                    Layout.preferredHeight: 68
                    Tr { code: "backBySaleSelectAll"; defaultMessage: "Выбрать все" }
                    onClicked: {
                        selectionModel.clearCurrentIndex()
                        handler.selectAll()
                    }
                }
                Item { Layout.preferredHeight: 8 }
                BasicButton {
                    Style.class: "backBySaleOk"
                    Layout.fillWidth: true
                    Layout.preferredHeight: 68
                    iconLayout.alignment: Qt.AlignTop | Qt.AlignRight
                    Tr { code: "backBySaleOk"; defaultMessage: "Возврат" }
                    icon: Style.parameter("icon.actionPanel.arrowBack")
                    onClicked: {
                        tableView.instantCommit()
                        handler.backBySale()
                    }
                }
            }
        }
    }

    footer: BottomPanel {}

    BackBySaleHandler { id: handler }
}
