import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import QtQuick.Shapes

import "../components"
import "view"

import Artix.Handlers
import Artix.Core
import Artix.Gui

/**
 * @brief Форма формиврования покупюрной ведомости
 */
BasicView {
    id: root

    property var bills: contextParams.billsFields
    property var coins: contextParams.coinsFields
    property list<var> fields: []
    property real sum: 0
    signal updateSum()
    onUpdateSum: {
        sum = 0
        for (let i = 0; i < fields.length; ++i)
            sum += fields[i].count * fields[i].value
    }

    // Фильтр для работы с клавиатурой
    property DialogEventFilter eventFilter: DialogEventFilter {
        onOk: handler.total(root.fields, root.sum)
        onCancel: handler.emitAction("COMMAND_ESC")
    }

    component InputComponent: BasicTextField {
        Style.class: "billsVerifyInput"

        required property int index
        required property var modelData
        readonly property real value: modelData.value
        readonly property var desc: modelData.desc
        required property bool isBill
        property int count: {
            if (text.length === 0)
                return 0
            return text
        }
        onCountChanged: updateSum()
        onTextChanged: {
            if (text.length === 0)
                text = "0"
            if (text.length == 2 && text.substring(0, 1) == "0")
                text = text.substring(1, 2)
        }

        Layout.preferredHeight: 56
        Layout.preferredWidth: 256
        padding: 8
        hideCursor: true
        fixedCursorPosition: true
        text: "0"
        down: pressed || focus
        horizontalAlignment: Qt.AlignRight

        BasicLabel {
            Style.class: "billsVerifyInputPlaceholder"
            anchors.left: parent.left
            anchors.bottom: parent.bottom
            anchors.margins: 12
            Tr { tr: desc }
        }
        validator: IntValidator { bottom: 0 }

        Component.onCompleted: fields.push(this)
    }

    contentItem: RowLayout {
        spacing: 0
        Rectangle {
            Layout.fillWidth: true
            Layout.fillHeight: true
            color: Style.parameter("color.modalBackground")
            // Поля ввода
            ColumnLayout {
                x: 24; y: 16
                spacing: 0
                BasicLabel {
                    Style.class: "billsVerifyTitle"
                    Tr { code: "billsverifyFormLabel"; defaultMessage: "Покупюрная ведомость" }
                }
                Item { Layout.preferredHeight: 24 }
                RowLayout {
                    spacing: 56
                    // Колонка купюр
                    ColumnLayout {
                        spacing: 8
                        BasicLabel {
                            Style.class: "billsVerifyCommon"
                            Tr { code: "billsVerifyFormBillLabel"; defaultMessage: "Купюры" }
                        }
                        Repeater {
                            model: root.bills
                            delegate: InputComponent { isBill: true }
                        }
                    }
                    // Колонка монет
                    ColumnLayout {
                        spacing: 8
                        BasicLabel {
                            Style.class: "billsVerifyCommon"
                            Tr { code: "billsVerifyFormCoinLabel"; defaultMessage: "Монеты" }
                        }
                        Repeater {
                            model: root.coins
                            delegate: InputComponent { isBill: false }
                        }
                    }
                }
                Item { Layout.preferredHeight: 24 }
                Item {
                    Layout.preferredWidth: root.width / 3
                    Layout.preferredHeight: sumLabel.height
                    BasicLabel {
                        id: sumLabel
                        Style.class: "billsVerifyCommon"
                        anchors.left: parent.left
                        Tr { code: "billsVerifyFormSumLabel"; defaultMessage: "Сумма" }
                    }
                    Line {
                        anchors {
                            left: sumLabel.right; right: sumValue.left; bottom: parent.bottom
                            bottomMargin: 5; leftMargin: 2; rightMargin: 2
                        }
                        strokeStyle: ShapePath.DashLine
                        dashPattern: [2, 3]
                        color: Style.parameter("color.gray")
                    }
                    BasicLabel {
                        id: sumValue
                        Style.class: "billsVerifyCommon"
                        anchors.right: parent.right
                        text: Utils.formatMoneyItem(root.sum)
                    }
                }
                Item { Layout.preferredHeight: 16 }
                Item {
                    Layout.preferredWidth: root.width / 3
                    Layout.preferredHeight: drawerLabel.height
                    visible: contextParams.showCashDrawerSum
                    BasicLabel {
                        id: drawerLabel
                        Style.class: "billsVerifyCommon"
                        anchors.left: parent.left
                        Tr { code: "billsVerifyFormDrawerSumLabel"; defaultMessage: "Сумма в д/я" }
                    }
                    Line {
                        anchors {
                            left: drawerLabel.right; right: drawerValue.left; bottom: parent.bottom
                            bottomMargin: 5; leftMargin: 2; rightMargin: 2
                        }
                        strokeStyle: ShapePath.DashLine
                        dashPattern: [2, 3]
                        color: Style.parameter("color.gray")
                    }
                    BasicLabel {
                        id: drawerValue
                        Style.class: "billsVerifyCommon"
                        anchors.right: parent.right
                        text: Utils.formatMoneyItem(SystemInfo.cashDrawerSum())
                    }
                }
            }
        }
        // Клавиатура и кнопки действий
        Item {
            Layout.fillHeight: true
            Layout.preferredWidth: 304
            TopPanel {
                anchors { left: parent.left; top: parent.top; right: parent.right }
                anchors.leftMargin: 24
                hideUserName: true
            }
            ColumnLayout {
                anchors { left: parent.left; right: parent.right; bottom: parent.bottom; margins: 24 }
                spacing: 0
                InlineKeyboard {
                    Layout.fillWidth: true
                    Layout.preferredHeight: 296
                    layoutType: "digits"
                }
                Item { Layout.preferredHeight: 16 }
                BasicButton {
                    Style.class: "billsVerifyCancel"
                    Layout.fillWidth: true
                    Layout.preferredHeight: 68
                    Tr { code: "billsVerifyFormCancelButton"; defaultMessage: "Назад" }
                    onClicked: handler.emitAction("COMMAND_ESC")
                }
                Item { Layout.preferredHeight: 8 }
                BasicButton {
                    Style.class: "billsVerifyOk"
                    Layout.fillWidth: true
                    Layout.preferredHeight: 68
                    iconLayout.alignment: Qt.AlignTop | Qt.AlignRight
                    Tr { code: "billsVerifyFormTotalButton"; defaultMessage: "Сохранить" }
                    onClicked: handler.total(root.fields, root.sum)
                }
            }
        }
    }

    BillsVerifyHandler { id: handler }

    footer: BottomPanel {}
}
