import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import QtQuick.Shapes

import "../components"
import "./view"

import Artix.Handlers
import Artix.Models
import Artix.Core
import Artix.Gui

/**
 * @brief Форма оплаты и сдачи
 */
BasicView {
    id: root
    property TextField inputField: null
    readonly property bool isSubtotal: Session.document.status === DocumentStatus.Payment

    property var card: Session.document.firstActiveCard
    // Фильтр и ресивер для работы с клавиатурой
    property TextField eventReceiver: inputField
    property FormEventFilter eventFilter: FormEventFilter {
        id: eventFilter
        onKeyboardData: (data, rawData, source) => handler.input(data, rawData, source)
        onCancel: handler.escCommand()
    }

    contentItem: RowLayout {
        spacing: 24
        // Левая часть - список позиций
        GroupBox {
            Style.class: "positionsListBackground"
            Layout.fillHeight: true
            Layout.fillWidth: true
            ColumnLayout {
                anchors.fill: parent
                spacing: 0
                // Информационные оповещения
                SingleNotification {
                    Layout.fillWidth: true
                    Layout.preferredHeight: 40
                    filter: Notification.Info
                    onClicked: topPanel.open()
                }
                // Шапка списка позиций
                PositionsListHeader {
                    id: positionListHeader
                    Layout.fillWidth: true
                    Layout.preferredHeight: 57
                    positionsCount: positions.count
                }
                // Список позиций
                PositionsList {
                    id: positions
                    Layout.fillWidth: true
                    Layout.fillHeight: true
                    interactive: false
                }
                // Дополнительная навигация
                ListNavigation {
                    target: positions
                    Layout.fillWidth: true
                    Layout.preferredHeight: 116
                    visible: Style.parameter("capacitiveTouchscreen") === "false"
                }
            }
        }
        // Правая часть - элементы управления
        Item {
            Layout.preferredWidth: Math.floor((screenWidth * 0.19 + 374) / 8) * 8
            Layout.fillHeight: true
            // Верхняя панель кассы
            TopPanel {
                id: topPanel
                anchors { left: parent.left; top: parent.top; right: parent.right }
            }
            ColumnLayout {
                anchors { left: parent.left; bottom: parent.bottom; right: parent.right; top: topPanel.bottom }
                anchors { rightMargin: 24; bottomMargin: 24; topMargin: errorNotification.visible ? 0 : 24 }
                spacing: 0
                // Критические оповещения
                SingleNotification {
                    id: errorNotification
                    Layout.fillWidth: true
                    Layout.preferredHeight: 40
                    filter: Notification.Error | Notification.Mode
                    onClicked: topPanel.open()
                }
                Item { Layout.preferredHeight: errorNotification.visible ? 4 : 0 }
                // Сумма чека/заголовок панели действий
                Item {
                    id: titleGroup
                    Layout.fillWidth: true
                    Layout.preferredHeight: 48
                    readonly property var subtotalTr: Utils.makeTr("documentSubtotalPayableSum", "К оплате")
                    readonly property var changeTr: Utils.makeTr("documentSubtotalChangeSum", "Сдача")
                    BasicLabel {
                        id: title
                        anchors.left: parent.left
                        anchors.bottom: parent.bottom
                        Style.class: "documentSubtotalTotalTitle"
                        Tr { tr: root.isSubtotal ? titleGroup.subtotalTr : titleGroup.changeTr }
                    }
                    BasicLabel {
                        id: totalValue
                        anchors.right: parent.right
                        anchors.verticalCenter: parent.verticalCenter
                        Style.class: "documentSubtotalTotalValue"
                        text: root.isSubtotal ? Utils.formatMoneyItem(Session.document.currentPayableSum)
                                              : Utils.formatMoneyItem(Session.document.changeSum)
                        horizontalAlignment: Text.AlignRight
                    }
                }
                Item { Layout.preferredHeight: errorNotification.visible ? 4 : 24 }
                StackView {
                    id: actionPanelStack
                    Layout.fillWidth: true
                    Layout.fillHeight: true
                    initialItem: ColumnLayout {
                        spacing: 10
                        Item {
                            Layout.fillWidth: true
                            property int earnRowHeigth: earnRow.visible ? 30 : 0
                            property int spentRowHeight: spentRow.visible ? 30 : 0
                            Layout.preferredHeight: earnRowHeigth + spentRowHeight
                            ColumnLayout {
                                anchors.fill: parent
                                spacing: 0
                                RowLayout {
                                    id: spentRow
                                    visible: card !== undefined && !Utils.moneyIsZero(card.spentSum)
                                    BasicLabel {
                                        Style.class: "bonusOperationName"
                                        Tr { code: "documentSubtotalSpentBonus"; defaultMessage: "Списано" }
                                    }
                                    Line {
                                        Layout.alignment: Qt.AlignBottom
                                        Layout.bottomMargin: 5
                                        Layout.fillWidth: true
                                        strokeStyle: ShapePath.DashLine
                                        dashPattern: [2, 3]
                                        color: Style.parameter("color.gray")
                                    }
                                    BasicLabel {
                                        Layout.preferredHeight: 20
                                        Style.class: "bonusOperationAmount"
                                        text: card !== undefined ? Utils.formatMoneyItem(card.spentSum) : ""
                                    }
                                }
                                RowLayout {
                                    id: earnRow
                                    visible: card !== undefined && !Utils.moneyIsZero(card.earnedSum)
                                    BasicLabel {
                                        Style.class: "bonusOperationName"
                                        Tr { code: "documentSubtotalEarnBonus"; defaultMessage: "Начислено" }
                                    }
                                    Line {
                                        Layout.alignment: Qt.AlignBottom
                                        Layout.bottomMargin: 5
                                        Layout.fillWidth: true
                                        strokeStyle: ShapePath.DashLine
                                        dashPattern: [2, 3]
                                        color: Style.parameter("color.gray")
                                    }
                                    BasicLabel {
                                        Layout.preferredHeight: 20
                                        Style.class: "bonusOperationAmount"
                                        text: card !== undefined ? Utils.formatMoneyItem(card.earnedSum) : ""
                                    }
                                }
                            }
                        }
                        // Заголовок
                        Item {
                            Layout.fillWidth: true
                            Layout.preferredHeight: 24
                            BasicLabel {
                                Style.class: "paymentsListTitleName"
                                anchors.left: parent.left
                                anchors.verticalCenter: parent.verticalCenter
                                Tr { code: "documentSubtotalPaymentSum"; defaultMessage: "Получено" }
                                visible: !Utils.moneyIsZero(Session.document.currentPaymentSum)
                            }
                            BasicLabel {
                                Style.class: "paymentsListTitleAmount"
                                anchors.right: parent.right
                                anchors.verticalCenter: parent.verticalCenter
                                text: Utils.formatMoneyItem(Session.document.currentPaymentSum)
                                visible: !Utils.moneyIsZero(Session.document.currentPaymentSum)
                            }
                        }
                        // Оплаты
                        PaymentList {
                            Layout.fillWidth: true
                            Layout.fillHeight: true
                        }
                        // Панель действий
                        ActionPanel {
                            id: actionPanel
                            Layout.fillWidth: true
                            Layout.preferredHeight: 380 + (screenHeight - 768) * (0.166 * rows)
                            context: root.isSubtotal ? Context.DocumentSubtotal
                                                     : Context.DocumentChange
                            onClicked: (actionCode) => { handler.processAction(actionCode) }
                        }
                    }
                    popEnter: null
                    popExit: null
                    pushEnter: null
                    pushExit: null
                    onCurrentItemChanged: {
                        if (actionPanelStack.depth === 1) {
                            inputField = null
                            return
                        }

                        inputField = currentItem.inputField
                        inputField.text = Utils.formatMoneyItem(Session.document.currentPayableSum)
                        inputField.selectAll()
                        currentItem.back?.connect(() => { actionPanelStack.pop() })
                        currentItem.commit?.connect((action, args) => {
                                                        actionPanelStack.pop()
                                                        handler.emitAction(action, args)
                                                   })
                    }
                }
            }
        }
    }
    // Нижняя панель кассы
    footer: BottomPanel {}

    // Коннектим сигналы из C++ движка
    Connections {
        id: connections
        target: engine

        // Показать панель оплаты бонусами
        function onShowBonusPanelExtension(action, defaultValue) {
            actionPanelStack.push("./view/actionPanelExtensions/BonusPaymentActionPanel.qml")
            actionPanelStack.currentItem.action = action
            inputField = actionPanelStack.currentItem.inputField
            inputField.text = Utils.formatMoneyItem(Session.document.firstActiveCard.pointsForSpend)
            inputField.selectAll()
        }
    }
    // Обработчик событий
    DocumentSubtotalHandler {
        id: handler
        onPanelExtension: (viewName, action) => {
                              actionPanelStack.push("./view/actionPanelExtensions/" + viewName + ".qml")
                              actionPanelStack.currentItem.action = action
                          }
        onDocumentChanged: positionListHeader.updateDocumentLabel(getDocumentInfo())
        Component.onCompleted: positionListHeader.updateDocumentLabel(getDocumentInfo())
    }
}
