import QtQuick
import QtQuick.Controls
import QtQuick.Layouts
import Qt.labs.qmlmodels

import "../components"
import "view"

import Artix.Handlers
import Artix.Core
import Artix.Gui
import Artix.Models

/**
 * @brief Форма выбора подарков
 */
BasicView {
    id: root
    implicitWidth: screenWidth
    implicitHeight: screenHeight
    readonly property bool fullScreen: true
    property DialogEventFilter eventFilter: DialogEventFilter {
        onOk: root.ok()
        onCancel: root.cancel()
    }

    GiftsChoiceModel {
        id: giftModel
        catalogCode: params.catalogCode
        maxQuantityVariant: params.maxQuantity
        minQuantityVariant: params.minQuantity
        property int top: 0
        property int count: 500
        Component.onCompleted: update(top, count)
    }
    readonly property bool toggleMode: giftModel.maxQuantity == 1 // Режим выбора одного подарка
    readonly property real quantityColumnWidth: toggleMode ? 0 : 280
    property int selectedRow: -1 // выделенная запись при toggleMode

    // Базовый делегат
    component BaseDelegate: GroupBox {
        Style.class: "giftsChoiceDelegate"
        padding: 16; leftPadding: 24; rightPadding: 24
        down: quantity > 0
        enabled: (giftModel.quantity != giftModel.maxQuantity || quantity > 0 || toggleMode) && valid
        Line {
            anchors.top: parent.bottom
            width: parent.width
            strokeWidth: 1
            color: Style.parameter("color.grayBorder")
        }
        interactive: toggleMode
        onClicked: {
            giftModel.minus(selectedRow)
            if (row === selectedRow)
                selectedRow = -1
            else {
                giftModel.plus(row)
                selectedRow = row
            }
        }
    }
    // Делегат имени подарка
    component NameDelegate: BaseDelegate {
        contentItem: BasicLabel {
            Style.class: "giftsChoiceItemName"
            verticalAlignment: Label.AlignVCenter
            text: display
        }
    }
    // Делегат выбора количества
    component QuantityDelegate: BaseDelegate {
        contentItem: RowLayout {
            BasicButton {
                Style.class: "giftsChoiceQuantityChange"
                Layout.preferredWidth: 64
                Layout.fillHeight: true
                icon: enabled ? Style.parameter("icon.minus") : Style.parameter("icon.minusDisabled")
                enabled: quantity > 0
                onClicked: giftModel.minus(row)
            }
            BasicTextField {
                Style.class: "giftsChoiceQuantityTextField"
                Layout.fillWidth: true
                Layout.fillHeight: true
                horizontalAlignment: TextField.AlignHCenter
                verticalAlignment: TextField.AlignVCenter
                text: display
                // Для запрета нажиматия на поле ввода
                // TODO: после обновления до Qt6.7+ попробовать использовать focusPolicy
                MouseArea { anchors.fill: parent }
            }
            BasicButton {
                Style.class: "giftsChoiceQuantityChange"
                Layout.preferredWidth: 64
                Layout.fillHeight: true
                icon: enabled ? Style.parameter("icon.plus") : Style.parameter("icon.plusDisabled")
                enabled: giftModel.quantity != giftModel.maxQuantity
                onClicked: giftModel.plus(row)
            }
        }
    }
    // Текст с информацией о количестве
    // В оригинальной форме было много вариантов этого сообщения,
    // но пока ограничемся более простым вариантом
    property string selectedQuantity: ""
    Tr on selectedQuantity {
        code: "giftsChoiceSelectedQuantity"
        defaultMessage: "выбрано %1 из %2"
        args: [giftModel.quantity, giftModel.maxQuantity]
    }
    property string minimumQuantity: ""
    Tr on minimumQuantity {
        code: "giftsChoiceMinimumQuantity"
        defaultMessage: "(минимум %1)"
        args: [giftModel.minQuantity]
    }

    contentItem: RowLayout {
        Rectangle {
            Layout.fillWidth: true
            Layout.fillHeight: true
            color: Style.parameter("color.modalBackground")
            ColumnLayout {
                anchors.fill: parent
                spacing: 0
                Item { Layout.preferredHeight: 16 }
                BasicLabel {
                    Style.class: "giftsChoiceTitle"
                    Layout.fillWidth: true
                    Layout.preferredHeight: 40
                    padding: 0; leftPadding: 24; rightPadding: 24
                    elide: Label.ElideRight
                    Tr { tr: params.title }
                }
                Item { Layout.preferredHeight: 8; visible: !toggleMode }
                BasicLabel {
                    Style.class: "giftsChoiceQuantityInfo"
                    Layout.fillWidth: true
                    padding: 0; leftPadding: 24
                    visible: !toggleMode
                    property bool showMinimumQuantity: giftModel.minQuantity > 0 && giftModel.minQuantity != giftModel.maxQuantity
                    text: selectedQuantity + (showMinimumQuantity ? (" " + minimumQuantity) : "")
                }
                Item { Layout.preferredHeight: 24 }
                HorizontalHeaderView {
                    id: headerView
                    Layout.fillWidth: true
                    Layout.preferredHeight: 30
                    model: giftModel
                    clip: true
                    resizableColumns: false
                    // Использование syncView приводит к задваиванию заголовков.
                    // Проверить после обновления Qt
                    columnWidthProvider: (column) => column === 1 ? quantityColumnWidth : width - quantityColumnWidth
                    rowHeightProvider: function(row) { return 30 }
                    interactive: false
                    delegate: BasicLabel {
                        Style.class: "giftsChoiceTableHeader"
                        padding: 0
                        leftPadding: 24
                        rightPadding: 24
                        verticalAlignment: Label.AlignVCenter
                        horizontalAlignment: column === 0 ? Qt.AlignLeft : Qt.AlignHCenter
                        Tr { tr: decoration }
                    }
                }
                Line {
                    Layout.fillWidth: true
                    Layout.preferredHeight: strokeWidth
                    strokeWidth: 2
                    color: Style.parameter("color.grayDark")
                }
                TableView {
                    id: tableView
                    Layout.fillWidth: true
                    Layout.fillHeight: true
                    model: giftModel
                    clip: true
                    contentWidth: width
                    columnWidthProvider: (column) => column === 1 ? quantityColumnWidth : width - quantityColumnWidth
                    rowHeightProvider: function(row) { return 88 }
                    rowSpacing: 1
                    resizableColumns: false
                    delegate: DelegateChooser {
                        role: "contentType"
                        DelegateChoice {
                            roleValue: GiftsChoiceModel.NameContent
                            delegate: NameDelegate {}
                        }
                        DelegateChoice {
                            roleValue: GiftsChoiceModel.QuantityContent
                            delegate: QuantityDelegate {}
                        }
                    }
                    // Дозагрузка данных при скролинге
                    onMovementEnded: {
                        if (atYBeginning) {
                            if (giftModel.top === 0)
                                return
                            let rowsShift = giftModel.count / 2
                            giftModel.top -= rowsShift
                            giftModel.update(giftModel.top, root.count)
                            updateModel()
                            forceLayout()
                            positionViewAtRow(rowsShift, TableView.AlignBottom)
                        }
                        if (atYEnd) {
                            if (giftModel.rowCount() < giftModel.count)
                                return
                            let rowsShift = giftModel.count / 2
                            giftModel.top += rowsShift
                            giftModel.update(giftModel.top, giftModel.count)
                            forceLayout()
                            positionViewAtRow(rowsShift - 1, TableView.AlignTop)
                        }
                    }
                    ScrollBar.vertical: ScrollBar {
                        id: scrollBar
                        Timer {
                            id: scrollTimer
                            interval: 2500
                            onTriggered: parent.active = false
                        }
                        onActiveChanged: scrollTimer.start()
                    }
                }
            }
        }
        Item {
            Layout.fillHeight: true
            Layout.preferredWidth: 304
            TopPanel {
                anchors { left: parent.left; top: parent.top; right: parent.right }
                anchors.leftMargin: 24
                hideUserName: true
            }
            ColumnLayout {
                anchors { left: parent.left; right: parent.right; bottom: parent.bottom; margins: 24 }
                spacing: 8
                BasicButton {
                    Style.class: "giftsChoiceCancel"
                    Layout.fillWidth: true
                    Layout.preferredHeight: 68
                    visible: params.canCancel
                    Tr { code: "cancelButton"; defaultMessage: "Отмена" }
                    onClicked: cancel()
                }
                BasicButton {
                    Style.class: "giftsChoiceOk"
                    Layout.fillWidth: true
                    Layout.preferredHeight: 68
                    iconLayout.alignment: Qt.AlignTop | Qt.AlignRight
                    visible: giftModel.selectedRequiredQuantity
                    Tr { code: "selectButton"; defaultMessage: "Выбрать" }
                    icon: Style.parameter("icon.actionPanel.arrow")
                    onClicked: ok()
                }
            }
        }
    }
    footer: BottomPanel {}

    DialogHandler { id: handler }
    function ok() {
        if (!giftModel.selectedRequiredQuantity)
            return
        handler.sendAnswer({"data": giftModel.quantityByBarcode})
    }
    function cancel() {
        if (!params.canCancel)
            return
        handler.sendAnswer()
    }
}
